import { Map } from 'immutable'
import Button from '@instructure/ui-buttons/lib/components/Button'

import ConfirmationModal from '../presenter'
import createShallowTestbed from '../../../../../tests/util/shallowTestbed'
import Modal from '../../Modal'

describe('Confirmation Modal', () => {
  const defaultProps = {
    closeModal: () => {},
    modalOpen: true,
    modalOptions: Map({})
  }
  const testbed = createShallowTestbed(ConfirmationModal, defaultProps)

  describe('modal header', () => {
    it('renders the given title', () => {
      const customTitle = 'Remove Item?'
      testbed.render({ modalOptions: Map({ title: customTitle }) })
      expect(testbed.getText('ModalHeader')).to.include(customTitle)
    })

    it('renders a default title if none given', () => {
      const standardTitle = 'Confirm'
      testbed.render()
      expect(testbed.getText('ModalHeader')).to.include(standardTitle)
    })

    it('renders a close button', () => {
      testbed.render()
      const closeButton = testbed.find('CloseButton')
      expect(closeButton).to.have.length(1)
      expect(closeButton.prop('children')).to.equal('Close')
    })
  })

  describe('modal footer', () => {
    it('renders the cancelText if given', () => {
      const customText = 'Dont do it'
      testbed.render({ modalOptions: Map({ cancelText: customText }) })
      expect(testbed.getText('ModalFooter')).to.include('Dont do it')
    })

    it('renders the continueText if given', () => {
      const customText = 'Do It'
      testbed.render({ modalOptions: Map({ continueText: customText }) })
      expect(testbed.getText('ModalFooter')).to.include('Do It')
    })
  })

  describe('main content', () => {
    it('renders at the size given', () => {
      testbed.render({ modalOptions: Map({ size: 'medium' }) })
      const modal = testbed.find(Modal)
      expect(modal.prop('size')).to.equal('medium')
    })

    it('renders with small size by default', () => {
      testbed.render()
      const modal = testbed.find(Modal)
      expect(modal.prop('size')).to.equal('small')
    })

    it('renders the text given', () => {
      const customText = 'Are you sure you wan\'t to do this?'
      testbed.render({ modalOptions: Map({ text: customText }) })
      expect(testbed.getText('ModalBody')).to.equal(customText)
    })
  })

  describe('actions', () => {
    const closeModalStub = sinon.stub()
    afterEach(() => {
      closeModalStub.resetHistory()
    })

    const buttonWithText = (text) => {
      return (n) => n.type() === Button && n.render().text() === text
    }

    it('calls cancelCb when you hit the cancel', () => {
      const cancelCbStub = sinon.stub()
      testbed.render({ modalOptions: Map({ onCancel: cancelCbStub }) })
      const cancelButton = testbed.findWhere(buttonWithText('Cancel'))
      testbed.click(cancelButton)
      expect(cancelCbStub).to.have.been.called()
    })

    it('calls continueCb when you hit continue', () => {
      const continueCbStub = sinon.stub()
      testbed.render({ modalOptions: Map({ onContinue: continueCbStub }) })
      const confirmButton = testbed.findWhere(buttonWithText('Confirm'))
      testbed.click(confirmButton)
      expect(continueCbStub).to.have.been.called()
    })

    it('calls closeModal when you hit the cancel', () => {
      testbed.render({ closeModal: closeModalStub })
      const cancelButton = testbed.findWhere(buttonWithText('Cancel'))
      testbed.click(cancelButton)
      expect(closeModalStub).to.have.been.called()
    })

    it('calls closeModal when you hit continue', () => {
      testbed.render({ closeModal: closeModalStub })
      const continueButton = testbed.findWhere(buttonWithText('Confirm'))
      testbed.click(continueButton)
      expect(closeModalStub).to.have.been.called()
    })
  })

  describe('a11y tests', () => {
    const testbedWithContext = createTestbed(withContext(ConfirmationModal), defaultProps)

    it('should meet a11y standards', (done) => {
      testbedWithContext.render()
      testbedWithContext.checkA11yStandards(done)
    })
  })
})
